import { promises } from "fs";
import { join } from "path";
import { FONT_MANIFEST, PAGES_MANIFEST, SERVER_DIRECTORY, SERVERLESS_DIRECTORY, APP_PATHS_MANIFEST } from "../shared/lib/constants";
import { normalizeLocalePath } from "../shared/lib/i18n/normalize-locale-path";
import { normalizePagePath } from "../shared/lib/page-path/normalize-page-path";
import { denormalizePagePath } from "../shared/lib/page-path/denormalize-page-path";
import { PageNotFoundError, MissingStaticPage } from "../shared/lib/utils";
export function getPagePath(page, distDir, serverless, dev, locales, appDirEnabled) {
    const serverBuildPath = join(distDir, serverless && !dev ? SERVERLESS_DIRECTORY : SERVER_DIRECTORY);
    let appPathsManifest;
    if (appDirEnabled) {
        appPathsManifest = require(join(serverBuildPath, APP_PATHS_MANIFEST));
    }
    const pagesManifest = require(join(serverBuildPath, PAGES_MANIFEST));
    try {
        page = denormalizePagePath(normalizePagePath(page));
    } catch (err) {
        console.error(err);
        throw new PageNotFoundError(page);
    }
    const checkManifest = (manifest)=>{
        let curPath = manifest[page];
        if (!manifest[curPath] && locales) {
            const manifestNoLocales = {};
            for (const key of Object.keys(manifest)){
                manifestNoLocales[normalizeLocalePath(key, locales).pathname] = pagesManifest[key];
            }
            curPath = manifestNoLocales[page];
        }
        return curPath;
    };
    let pagePath;
    if (appPathsManifest) {
        pagePath = checkManifest(appPathsManifest);
    }
    if (!pagePath) {
        pagePath = checkManifest(pagesManifest);
    }
    if (!pagePath) {
        throw new PageNotFoundError(page);
    }
    return join(serverBuildPath, pagePath);
}
export function requirePage(page, distDir, serverless, appDirEnabled) {
    const pagePath = getPagePath(page, distDir, serverless, false, undefined, appDirEnabled);
    if (pagePath.endsWith(".html")) {
        return promises.readFile(pagePath, "utf8").catch((err)=>{
            throw new MissingStaticPage(page, err.message);
        });
    }
    return require(pagePath);
}
export function requireFontManifest(distDir, serverless) {
    const serverBuildPath = join(distDir, serverless ? SERVERLESS_DIRECTORY : SERVER_DIRECTORY);
    const fontManifest = require(join(serverBuildPath, FONT_MANIFEST));
    return fontManifest;
}

//# sourceMappingURL=require.js.map